﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ZoneSpawner : MonoBehaviour
{
	// Specification for a win zone or a lose zone.  Where it's located, how big it is.
	private class ZoneInfo
	{
		public Vector3 position;
		public Vector3 scale;
		public bool isMoving;

		public ZoneInfo (float x, float y, float width, float height, bool isMoving)
		{
			this.position = new Vector3 (x, y, Z_LAYER);
			this.scale = new Vector3 (width, height, 1.0f);
			this.isMoving = isMoving;
		}
	}

	// The configuration of an entire level, specifically where its win zones and lose zones are.
	private class LevelConfig
	{
		public List<ZoneInfo> winZones;
		public List<ZoneInfo> loseZones;

		public LevelConfig (List<ZoneInfo> winZones, List<ZoneInfo> loseZones)
		{
			this.winZones = winZones;
			this.loseZones = loseZones;
		}
	}

	public GameObject winningZonePrefab;
	public GameObject losingZonePrefab;

	// Will appear behind the balls, and in front of the backdrop.
	private const float Z_LAYER = 1.0f;

	// Level data.  Data for level 1 is stored at index 1, etc.   When this array runs out, the player's won!
	private List<LevelConfig> levelConfigs = new List<LevelConfig> () {
		// Level 0
		new LevelConfig (
			new List<ZoneInfo> (),  // Win zones.  Empty for level 0.
			new List<ZoneInfo> ()  // Lose zones.  Empty for level 0.
		),
		// Level 1
		new LevelConfig (
			new List<ZoneInfo> () {  // Win zones.
				new ZoneInfo (0.0f, -4.5f, 2.0f, 1.0f, false),
			},
			new List<ZoneInfo> () {  // Lose zones.
			}
		),
		// Level 2
		new LevelConfig (
			new List<ZoneInfo> () {  // Win zones.
				new ZoneInfo (0.0f, -4.5f, 2.0f, 1.0f, false),
			},
			new List<ZoneInfo> () {  // Lose zones.
				new ZoneInfo (3.0f, -1.0f, 6.0f, 1.0f, false),
			}
		),
		// Level 3
		new LevelConfig (
			new List<ZoneInfo> () {  // Win zones.
				new ZoneInfo (-4.0f, -4.5f, 1.0f, 1.0f, true),
			},
			new List<ZoneInfo> () {  // Lose zones.
				new ZoneInfo (-4.0f, 0.0f, 1.0f, 1.0f, true),
			}
		),
	};

	private BallSpawner ballSpawner;

	private Score score;
	private int level = 0;
	private List<GameObject> activeZones = new List<GameObject> ();
	private bool gameStarted = false;

	// Use this for initialization
	void Start ()
	{
		// For turning off the balls at the end.
		ballSpawner = GameObject.Find ("Ball Spawner").GetComponent<BallSpawner> ();

		// Initialize score.
		score = GameObject.Find ("Score Text").GetComponent<Score> ();
		score.ClearPercentage ();
		score.SetInstructions ();
	}

	// Update is called once per frame
	void Update ()
	{
		// ENTER starts the game, if we're in a state waiting for it to be pressed.
		if (!gameStarted && (Input.GetKey (KeyCode.Return) || Input.GetKey (KeyCode.KeypadEnter))) {
			gameStarted = true;

			level = 1;
			score.SetLevel (level);
			score.ClearPercentage ();
			score.ContinuouslyShowPercentage ();

			CreateZones ();
			ballSpawner.StartBallSpawns ();
		}

		if (score.GetPercentage () >= 100.0f) {
			// Move on to the next level.
			ClearZones ();

			level++;
			score.SetLevel (level);
			score.ClearPercentage ();

			if (level >= levelConfigs.Count) {
				gameStarted = false;
				score.SetYouWin ();
				ballSpawner.StopBallSpawns ();
			} else {
				CreateZones ();
			}
		}
	}

	// Create Zones for the current level.
	void CreateZones ()
	{
		LevelConfig levelConfig = levelConfigs [level];
		foreach (ZoneInfo zone in levelConfig.winZones) {
			SpawnZone (winningZonePrefab, zone);
		}
		foreach (ZoneInfo zone in levelConfig.loseZones) {
			SpawnZone (losingZonePrefab, zone);
		}
	}

	void SpawnZone (GameObject prefab, ZoneInfo zone)
	{
		var obj = Object.Instantiate (prefab, zone.position, Quaternion.identity);
		obj.transform.localScale = zone.scale;
		ZoneMover zoneMover = obj.GetComponent<ZoneMover> ();
		zoneMover.SetMoving (zone.isMoving);
		activeZones.Add (obj);
	}

	// Remove all Zones in the current level.
	void ClearZones ()
	{
		foreach (GameObject obj in activeZones) {
			Destroy (obj);
		}
		activeZones.Clear ();
	}
}
